import numpy as np
import matplotlib.pyplot as plt
from scipy.io import loadmat
from scipy.sparse.linalg import svds
from lrgd import lrgd
from lrsgd import lrsgd

# load data
data = loadmat("mnistSubset.mat")
X = data['X']
trueLabels = data['trueLabels'][:,0]

# only consider digits 1 and 2 (labels 2 and 3)
K = 2
Xtrain = np.array([]).reshape(784, 0)
ytrain = np.array([]).reshape(0, 1)
Xtest = np.array([]).reshape(784, 0)
ytest = np.array([]).reshape(0, 1)
for kk in range(1,K+1):
    Xk = X[:, np.argwhere(trueLabels==kk+1)[:,0]]
    Xtrain = np.append(Xtrain, Xk[:, :100], axis=1)
    Xtest = np.append(Xtest, Xk[:, 100:], axis=1)
    ytrain = np.append(ytrain, kk*np.ones([100, 1]), axis=0)
    ytest = np.append(ytest, kk*np.ones([100, 1]), axis=0)

# make labels +1/-1
ytrain = (2*(ytrain - 1) - 1)[:,0]
ytest = (2*(ytest - 1) - 1)[:,0]

## YOUR CODE
# reduce dimension of data using PCA and plot

## Part b
# train using standard gradient descent
numIter = 20
lam = 0.001
wgd, gdCost = lrgd(Xtrain, ytrain, lam, numIter)

# plot separator
minX = np.min(Xtrain[0,:])
maxX = np.max(Xtrain[0,:])
xx = np.linspace(minX,maxX,100)
yy = -wgd[0]*xx/wgd[1]
plt.plot(xx, yy, label='GD separator')

# compute train and test error

## Part c
# train using stochastic gradient descent
wsgd, sgdCost = lrsgd(Xtrain, ytrain, lam, numIter)

# plot separator
yy = -wsgd[0]*xx/wsgd[1]
plt.plot(xx, yy, label='SGD separator')
plt.legend()
plt.show()

# compute train and test error

# plot cost vs iteration
plt.figure()
plt.plot(np.arange(numIter), gdCost, label='GD')
plt.plot(np.arange(numIter), sgdCost, label='SGD')
plt.title('Logistic Loss')
plt.legend()
plt.show()
